/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package example.mmademo;

import javax.microedition.content.*;


/**
 * The handler for ContentHandler requests.
 *
 */
public class VideoHandler implements Runnable {
    /** The invoking VideoAlbum midlet for context. */
    VideoTest videotest;

    /** The Handler being serviced. */
    ContentHandlerServer handler;

    /** The current Invocation being processed. */
    Invocation request;

    /** True if this handler should be stopping. */
    boolean stopping;

    /**
     * Create a new instance of VideoHandler and start the thread
     * to service it.
     */
    public VideoHandler(VideoTest videotest) {
        this.videotest = videotest;

        try {
            handler = Registry.getServer(videotest.getClass().getName());
        } catch (ContentHandlerException che) {
            // handle exception
        }

        // check for a pending request;
        request = handler.getRequest(false);
        new Thread(this).start();
    }

    /**
     * Return the current invocation.
     * @return the current Invocation.
     */
    public Invocation current() {
        return request;
    }

    /**
     * Stop this handler as soon a possible.
     * Any current request will be cancelled.
     */
    public void stop() {
        stopping = true;
    }

    /**
     * Handle incoming requests.
     */
    public void run() {
        do {
            if (request != null) {
                videotest.newRequest(request.getURL());
            }

            // Look for a new request
            Invocation inv = null;

            while (!stopping && (inv == null)) {
                inv = handler.getRequest(true);
            }

            // Have new request; need to finish the previous one, if...
            if (request != null) {
                // If previous invocation was not finished; cancel it
                if (handler.finish(request, Invocation.CANCELLED)) {
                    // Must exit to deliver the response
                    videotest.destroyApp(false);
                    videotest.notifyDestroyed();
                }

                request = null;
            }

            // Make the new invocation current and notify the VideoTest
            request = inv;
        } while (!stopping);
    }

    /**
     * Finish the current invocation.
     */
    public void finish(boolean success) {
        if (request != null) {
            if (handler.finish(request, success ? Invocation.OK : Invocation.CANCELLED)) {
                // Must exit to deliver the response
                videotest.destroyApp(false);
                videotest.notifyDestroyed();
            }

            request = null;
        }
    }
}
